# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: DependencyTestProto.proto

import sys
_b=sys.version_info[0]<3 and (lambda x:x) or (lambda x:x.encode('latin1'))
from google.protobuf import descriptor as _descriptor
from google.protobuf import message as _message
from google.protobuf import reflection as _reflection
from google.protobuf import symbol_database as _symbol_database
from google.protobuf import descriptor_pb2
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from . import NestedTestProto_pb2 as NestedTestProto__pb2
from . import PublicTestProto_pb2 as PublicTestProto__pb2
try:
  TestProto__pb2 = PublicTestProto__pb2.TestProto__pb2
except AttributeError:
  TestProto__pb2 = PublicTestProto__pb2.TestProto_pb2


DESCRIPTOR = _descriptor.FileDescriptor(
  name='DependencyTestProto.proto',
  package='tests.integration.serialization.data',
  syntax='proto3',
  serialized_pb=_b('\n\x19\x44\x65pendencyTestProto.proto\x12$tests.integration.serialization.data\x1a\x15NestedTestProto.proto\x1a\x15PublicTestProto.proto\"\x98\x01\n\x11\x44\x65pendencyMessage\x12K\n\x0enested_message\x18\x01 \x01(\x0b\x32\x33.tests.integration.serialization.data.NestedMessage\x12\x11\n\tis_active\x18\x02 \x01(\x08\x12#\n\rtest_messsage\x18\x03 \x01(\x0b\x32\x0c.TestMessageB.\n,io.confluent.kafka.serializers.protobuf.testb\x06proto3')
  ,
  dependencies=[NestedTestProto__pb2.DESCRIPTOR,PublicTestProto__pb2.DESCRIPTOR,])




_DEPENDENCYMESSAGE = _descriptor.Descriptor(
  name='DependencyMessage',
  full_name='tests.integration.serialization.data.DependencyMessage',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='nested_message', full_name='tests.integration.serialization.data.DependencyMessage.nested_message', index=0,
      number=1, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='is_active', full_name='tests.integration.serialization.data.DependencyMessage.is_active', index=1,
      number=2, type=8, cpp_type=7, label=1,
      has_default_value=False, default_value=False,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='test_messsage', full_name='tests.integration.serialization.data.DependencyMessage.test_messsage', index=2,
      number=3, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=114,
  serialized_end=266,
)

_DEPENDENCYMESSAGE.fields_by_name['nested_message'].message_type = NestedTestProto__pb2._NESTEDMESSAGE
_DEPENDENCYMESSAGE.fields_by_name['test_messsage'].message_type = TestProto__pb2._TESTMESSAGE
DESCRIPTOR.message_types_by_name['DependencyMessage'] = _DEPENDENCYMESSAGE
_sym_db.RegisterFileDescriptor(DESCRIPTOR)

DependencyMessage = _reflection.GeneratedProtocolMessageType('DependencyMessage', (_message.Message,), dict(
  DESCRIPTOR = _DEPENDENCYMESSAGE,
  __module__ = 'DependencyTestProto_pb2'
  # @@protoc_insertion_point(class_scope:tests.integration.serialization.data.DependencyMessage)
  ))
_sym_db.RegisterMessage(DependencyMessage)


DESCRIPTOR.has_options = True
DESCRIPTOR._options = _descriptor._ParseOptions(descriptor_pb2.FileOptions(), _b('\n,io.confluent.kafka.serializers.protobuf.test'))
# @@protoc_insertion_point(module_scope)
