# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: PublicTestProto.proto

import sys
_b=sys.version_info[0]<3 and (lambda x:x) or (lambda x:x.encode('latin1'))
from google.protobuf import descriptor as _descriptor
from google.protobuf import message as _message
from google.protobuf import reflection as _reflection
from google.protobuf import symbol_database as _symbol_database
from google.protobuf import descriptor_pb2
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from . import TestProto_pb2 as TestProto__pb2

from .TestProto_pb2 import *

DESCRIPTOR = _descriptor.FileDescriptor(
  name='PublicTestProto.proto',
  package='tests.integration.serialization.data',
  syntax='proto3',
  serialized_pb=_b('\n\x15PublicTestProto.proto\x12$tests.integration.serialization.data\x1a\x0fTestProto.protoP\x00\x62\x06proto3')
  ,
  dependencies=[TestProto__pb2.DESCRIPTOR,],
  public_dependencies=[TestProto__pb2.DESCRIPTOR,])



_sym_db.RegisterFileDescriptor(DESCRIPTOR)


# @@protoc_insertion_point(module_scope)
