{
    "name": "basic",
    "type": "record",
    "doc": "basic schema for tests",
    "namespace": "python.test",
    "fields": [
        {
            "name": "number",
            "doc": "age",
            "type": [
                "long",
                "null"
            ]
        },
        {
            "name": "name",
            "doc": "a name",
            "type": [
                "string"
            ]
        }
    ]
}
