syntax = "proto3";

import "google/protobuf/timestamp.proto";

option java_package = "io.confluent.kafka.serializers.protobuf.test";

// Wrapper message for `Timestamp`.
message TimestampValue {
  // The bytes value.
  google.protobuf.Timestamp value = 1;
}
